FORWARD_FUNCTION BOOTMSINI

PRO boottran,tve1,tve2,tve3,tve4,tve5,tve6,tele=tele,bestpar=bestpar,date=date,ntran=ntran,Tc_planet=Tc_planet $
             ,bootpar=bootpar,bootnewpar=bootnewpar,bootoffset=bootoffset,nboot=nboot, bootse=bootse $
             ,bootfile=bootfile,debug=debug,sigmaTcs=sigmaTcs,bestTcs=bestTcs $
             ,Mstar=mstar,sigma_m=sigma_m,nosave=nosave,bootout=boot,data=data,noprint=noprint,jitter=jitter $
             ,regularjd=regularjd,coords=coords,_extra=extra

;+
;; boottran,tve1,tve2,tve3,tve4,tve5,tve6,tele=tele,bestpar=bestpar,date=date,ntran=ntran $
;;             ,bootpar=bootpar,bootnewpar=bootnewpar,bootoffset=bootoffset $
;;             ,bootfile=bootfile,debug=debug,sigmaTcs=sigmaTcs,bestTcs=bestTcs $
;;             ,Mstar=mstar,sigma_m=sigma_m,nosave=nosave,bootout=boot,data=data,noprint=noprint,jitter=jitter
;;
;; Procedure for calculating uncertainties via bootstrap (bootpar.pro),
;; and calculate transit time window (mid-point and uncertainties).
;; Meant to be used with bootpar.pro and rvlin package by Jason T. Wright.
;; For rvlin package, please see exoplanets.org.
;; 
;; INPUT PARAMETERS / KEYWORDS:
;; tve1, tve2 .... tve6 (structure) Input structure. Must have tags JD,
;;                                  MNVEL and ERRVEL.
;;                     (array)     tve1 = JD array, tve2 = MNVEL array,
;;                                 tve3 = ERRVEL array.
;; tele (optional array) Telescope labels.
;; bestpar (array) Input best fitted parameters, either 7 or 14 elements
;;                 for 1 or 2 planet cases, respectively.
;; bootpar, bootnewpar, bootoffset  (arrays) If not given, they return
;;                 bootstrap results. See bootpar.pro. dblarr[nboot,npars]
;;                 If given, they are bootstrap results to be used.
;; nboot (optional long/int) number of bootstrap steps / sampling size.
;; bootfile (string) File name for the save file to store bootpar,
;;                   bootnewpar and bootoffset. Can be existing file
;;                   that contains bootpar etc already.
;; date (string) Eight numbers, denoting starting date to give transit
;;               window. e.g. '20110228'. Next 5 transit times will be
;;               output after the given date. Default is current date.
;; ntran (int)   int > 0, number of transit mid-points you want.
;; Tc_planet (int)  Which planet you want Tc for? Default is the first
;;               one (planet=1). Only makes sense for a new run!!! 
;; mstar (double, optional) Host star mass, in solar mass. If given,
;;                          m*sin(i) and semi-major axis a will be calculated.
;; sigma_m (double, optional) Host star mass uncertainty, in solar
;;                            mass. If give, uncertainties on m*sin(i)
;;                            and a are calculated.
;; jitter (double) Stellar jitter in m/s.
;; debug (keyword) if /debug, then bootstrap sampling distribution of
;;                 Tc will be displayed.
;; sigmaTcs (double arr) to pass out the sigmas of Tc's,
;;                       bootstrap uncertainties.
;; bestTcs (double arr) to pass out the Tc's from best fitted
;; parameters.
;; bootse (double arr) to pass out the secondary eclipses from best fitted parameters.
;; nosave (keyword) if /nosave, then results won't be saved in bootfile.
;; bootout (structure) Structure containing everything that's ever
;;                  calculated here.
;; data (structure) Save the data used for this bootstrap effort.
;; noprint (keyword) if /noprint, then results won't be output
;; on screen.
;; coords (2-element double array) target [ra,dec] in degrees, for
;;                                 converting BJD into JD or JD into BJD.
;; regularjd (keyword) If /regularjd, then if coords are provided,
;;                     reported Tc will be converted to BJD.
;; _extra (inherited keywords) keywords or parameters to be inherited
;;                             by bootpar then rv_fit_mp.
;;
;;
;; OUTPUT:
;; data (structure) contains your input data.
;; boot (structure) contains your best-fit, uncertainties, bootstrap
;;                  replicates etc.
;;
;; Examples:
;; 1. One telescope situation:
;; (1) You have JD, Vel and Err stored in arrays:
;; IDL> boottran, jd, vel, err, bestpar=bestpar,bootfile='mybootpar.sav', date='20110101'
;; (2) You have JD, vel and err saved in a structure called TVE, which has tve.jd, tve.mnvel, tve.errvel.
;; (Note: tve may also contain other tags, but only these 3 are relevant).
;; IDL> boottran,tve,bestpar=bestpar,bootfile='mybootpar.sav', date='20110101'

;; 2. Multiple telescope situation:
;; (1) You have JD, Vel, Err and telescope-markers(tele) stored in arrays:
;; IDL> boottran, jd, vel, err, bestpar=bestpar,tele=tele, bootfile='mybootpar.sav', date='20110101'
;; (2) You have JD, vel and err saved in several structures called tve1, tve2...(up to tve6),
;; each of which has tve.jd, tve.mnvel, tve.errvel.
;; (again, tve can contain other tags.)
;; IDL> boottran,tve1,tve2,...,tve6,bestpar=bestpar,bootfile='mybootpar.sav', date='20110101'

;; 3. You have previously run the code and just want to compute transit times after a different date, or just want more of them.
;; Your old run saved everything in bootfile='mybootpar.sav', just like the example above.
;; IDL> boottran,bootfile='mybootpar.sav',date='20111205',ntran=100
;;  
;;
;; Written by Sharon X. Wang, Feb 28 2011
;;
;; ----------------------------------------------------------------------------
;;          Version 1.1   Modification History
;; ----------------------------------------------------------------------------  
;; Modified to accept arrays for jd,vel,err,tele=tele for multiple
;; telescope situation. SXW Mar 2011
;;  
;; Modified to adopt calculation on m*sin(i) and semi-major axis a and
;; their uncertainties. Accept keywords mstar and sigma_m now. SXW Apr 2011
;;
;; Adjusted output save file and modify features to enable a wrapper
;; around this. SXW May 2011
;;
;; Added stellar jitter. SXW May 11 2011
;;
;; Fixed M*sin(i) calculation (now assume sini=1 and solve Mp
;; iteratively, using function bootmsini.pro).
;; Added BJD and JD conversion, depends on what you give as JD
;; (regularjd and coords input keywords/parameters). SXW Dec 13 2011
;;
;; Added keyword inheritance _extra for bootpar to pass on to
;; rv_fit_mp. SXW Jan 30 2011
;;   
;; Fixed bugs in sigma_msini and sigma_a calculation, including
;; (finally) a correct equation for sigma_msini and stopping using
;; the assumption that Bootmsini << Mstar anywhere. SXW Mar 27 2012
;;
;; Added nboot parameter to enable increase in bootstrap step.
;; SXW Mar 2012
;;
;; Merged with EOD version including secondary eclipse calculation
;; JTW SXW April 23, 2012
;;
;; Fixed bug that caused crashes due to nonexistent Te variables JTW Jun 5 2012
;; ----------------------------------------------------------------------------  
;-

  ; Parameter input check
  if 1-keyword_set(bootpar) and 1-keyword_set(bootnewpar) and 1-keyword_set(bootfile) $
     and n_params() lt 1 then begin
     print,'%% BOOTTRAN: TVE structure is not given, nor are bootpar and bootnewpar.'
     stop
  endif 
  
  ; Check the date, convert into JD (UTC)
  if keyword_set(date) then begin
     if strlen(date) ne 8 then begin
        print,'%% BOOTTRAN: Date must be eight digit in string format.'
        print,'%% BOOTTRAN: Press any key to use TODAY in UTC, q to quit.'
        r=get_kbrd()
        if r eq 'q' then retall else jdtoday=double(systime(/julian,/utc))
     endif else begin
        year=double(strmid(date,0,4))
        month=double(strmid(date,4,2))
        day=double(strmid(date,6,2))
        if year lt 1900 or month gt 12 or day gt 31 then begin
           print,'%% BOOTTRAN: Wrong date input. Format:'
           print,'             20110228 for Feb 28 2011.'
           stop
        endif
        jdtoday = double(julday(month,day,year,0.)) ; double precision by given hour=0.
     endelse 
  endif else begin
     if 1-keyword_set(noprint) then print,'%% BOOTTRAN: Date not specified. Using TODAY in UTC:'
     if 1-keyword_set(noprint) then print,systime(0,/utc)
     jdtoday=double(systime(/julian,/utc))
  endelse 

;------------------------------------------------------------------------------------------------
  ; Need bootstrap?
  newbootfile=0
  if keyword_set(bootfile) then begin
     if findfile(bootfile) eq '' then newbootfile = 1
  endif else begin
     if n_elements(bootpar) lt 1 then newbootfile = 1
  endelse

  if newbootfile eq 1 then begin
     ; A new run!
     nplanet = n_elements(bestpar)/7
     if 1-keyword_set(Tc_planet) then Tc_planet = 1
     if 1-keyword_set(noprint) then print,''
     if 1-keyword_set(noprint) then print,'%% BOOTTRAN: Bootstrapping...'
     ; Bootstrap Core Code
     bootpar,tve1,tve2,tve3,tve4,tve5,tve6,bestpar=bestpar, $ 
             bootpar=bootpar,bootnewpar=bootnewpar,bootse = bootse,bootoffset=bootoffset,tele=tele, $
             Tc_planet=Tc_planet,jitter=jitter, _extra=extra, nboot=nboot
  endif else begin
     ; Read in or Use old results
     if n_elements(bootpar) lt 1 then begin
        if 1-keyword_set(noprint) then print,'%% BOOTTRAN: Reading in old bootstrap results from '+bootfile
        restore,bootfile,/v
        if keyword_set(boot) then begin ; new format bootfile?
           bootse = boot[0].bootse
           bestpar = boot[0].bestpar
           bootpar = boot[0].bootpar
           bootnewpar = boot[0].bootnewpar
           bootoffset = boot[0].bootoffset
           Tc_planet = boot[0].Tc_planet
           ; If new stellar mass and error bar is passed in, use the new ones:
           if n_elements(mstar)   eq 0 and datatype(boot[0].mstar)   ne 'STR' then mstar   = boot[0].mstar
           if n_elements(sigma_m) eq 0 and datatype(boot[0].sigma_m) ne 'STR' then sigma_m = boot[0].sigma_m
        endif else begin
           Tc_planet = 1 ; old style
        endelse 
     endif  
  endelse 

  if 1-keyword_set(bestpar) then begin
     print,'%% BOOTTRAN: I absolutely need bestpar to continue!'
     stop
  endif
  
;------------------------------------------------------------------------------------------------
; Begin output and uncertainty calculation based on bootstrap sampling distribution
  
  if 1-keyword_set(noprint) then print,''

                                ; First for the regular parameters,
                                ; bestpar(nplanet*7), mass function,
                                ; msini, and a:
  if keyword_set(bootpar) then begin
     nplanet = n_elements(bootpar[0,*])/7
     ;
     ; Begin calculation of msini and a, if given mstar (and sigma_m)
     if keyword_set(mstar) then begin ; msini and a can be derived
        G = double(6.67428e-8)        ; cgs gravitational constant
        PI = !dpi                     ; double precision pi
        Msun = double(1.98892e33)     ; solar mass in g
        Mjup = double(1.8986e30)      ; jupiter mass in g
        Mearth = double(5.9737e27)    ; earth mass in g
        AU = double(1.495978707e13)   ; AU in cm
        ; WHAT WE WANT:
        massfunc = dblarr(nplanet) ; massfunction, in g
        msini = dblarr(nplanet)    ; msini, in Jupiter Mass
        a = dblarr(nplanet)        ; semi-major axis a, in AU
        if n_elements(sigma_m) ge 1 then begin
           sigma_massf = dblarr(nplanet) ; sigma(massfunc), in g
           sigma_msini = dblarr(nplanet) ; sigma(msini) in Jupiter Mass
           sigma_a = dblarr(nplanet)     ; sigma(a) in AU
        endif
        
        ; Begin calculation of msini and a etc.
        for i=0,nplanet-1 do begin
           Psec = double( bestpar[i*7]*24.0*3600.0 ) ; period in seconds
           K = double( bestpar[i*7+4]*100.0 ) ; cm/s
           ecc = bestpar[i*7+2]
           massfunc[i] = double( Psec*K*K*K*(1-ecc*ecc)^1.5 / (2.0*PI*G) ) ; mass function in g
           
                                ; True msini value with assumption
                                ; that sini = 1, solving for Mplanet recursively
           msini[i] = bootmsini(mstar,massfunc[i]) ; using the msini.pro function, in Jupiter mass
           logA = double( (alog(Psec*Psec*G*mstar/(4.0*PI*PI))+alog(Msun))/3.0-alog(AU) ) ; with semi-major axis a in AU
           a[i] = exp(logA) ; a in AU
           
           ; Given sigma_m, uncertainties on msini and a are possible to get:
           if n_elements(sigma_m) ge 1 then begin
              ; sigma(mass function) via sampling distribution:
              massfs = double( bootpar[*,i*7]*24.0*3600.0  * $
                               (bootpar[*,i*7+4]*100.0)^3  * $ ; mind the units!!!! P in sec, K in cm/s
                               (1-bootpar[*,i*7+2]*bootpar[*,i*7+2])^1.5/2.0/PI/G ) 
              sigma_massf[i] = robust_sigma(massfs)
              
              ; sigma(msini) via propagation of errors:
              mm = msini[i] ; easier to type, in Jupiter mass
              sigmamm1 = robust_sigma(bootmsini(mstar,massfs)) ; dmm/df*sigma(f), in M_Jup, this is an ensemble of msini if mstar = const.
              sigmamm2 = sigma_m*Msun/Mjup/(1.5*sqrt(mstar*Msun/massfunc[i])-1.0) ; dmm/dmstar*sigma(mstar), in M_Jup, dmm/dmstar = 1/(dmstar/dmm|f=const)
              sigma_msini[i] = sqrt(sigmamm1^2 + sigmamm2^2)
              
              ; sigma(a) via propagation of errors:
              siga31 = robust_sigma( double( bootpar[*,i*7]*bootpar[*,i*7] ) )*(24.0*3600.0)^2 ; P in seconds!!! This is ensemble of P^2.
              siga31 *= ((mstar*Msun+msini[i]*Mjup)*G/4.0/PI/PI)
              siga32 = Psec*Psec*G/4.0/PI/PI*( sqrt( (sigma_m*Msun)^2 + (sigma_msini[i]*Mjup)^2 ) )
              sigma_a[i] = 1.0/(a[i]*a[i]*3.0) * sqrt(siga31^2 + siga32^2) / (AU^3) ; from sigma(a^3) to sigma(a), in AU
           endif
        endfor
     endif
     ;
     ; Begin calculation of uncertainties for bestpar
     sigma_par = dblarr(n_elements(bestpar))
     for i=0,n_elements(bestpar)-1 do begin
        sigma_par[i] = robust_sigma(bootpar[*,i])
     endfor 
     ;
     ; Output 
     if 1-keyword_set(noprint) then begin
        print,'Best fit +/- Bootstrap uncertainties:'
        bootpardef = ['Period [day]','Tp [JD-2440000]','e','omega [deg]','K [m/s]','gamma [m/s]','dv/dt [m/s/day]']
        otherParDef = ['Mass Function [g]','m*sin(i) [M_Jupiter]','semi-major axis [AU]']
        for i=0,nplanet-1 do begin
           print,'Planet # '+strtrim(i+1,2)+':'
           for j=0,6 do begin
              print,format='(A-15,":",G16.8E2,"  +/-  ",G-16.8E2)', $
                    bootpardef[j], bestpar[i*7+j], sigma_par[i*7+j]
           endfor
           if keyword_set(mstar) then begin
              otherPar = [massfunc[i],msini[i],a[i]] ; mass function, msini, a of this planet # i
              if n_elements(sigma_m) lt 1 then begin
                 sigmas = dblarr(nplanet*3)
                 print,'!!! Need sigma(Mstar) to compute uncertainties on parameters below !!!'
              endif else begin
                 sigmas = [sigma_massf[i],sigma_msini[i],sigma_a[i]]
              endelse 
              for j=0,2 do begin
                 print,format='(A-22,":",G16.8E2,"  +/-  ",G-16.8E2)', $
                       otherParDef[j], otherPar[j], sigmas[j]
              endfor 
           endif else begin
              print,'!!! Need Mstar to compute m*sin(i) and semi-major axis !!!'
           endelse 
        endfor
        print,''
     endif 
  endif
  
;------------------------------------------------------------------------------------------------
  ; Then for the transit times:
  if keyword_set(bootnewpar) then begin
     tcp = Tc_planet-1 ; index for the planet whose Tcs are being calculated
     nplanet = n_elements(bestpar)/7
     if 1-keyword_set(ntran) then begin
        if 1-keyword_set(noprint) then $
           print,'%% BOOTTRAN: Number of transit mid-points not given. Calculating next 5 transits...'
        ntran = 5
     endif 
     ; (1) Tc0 from best fitted parameters
     newpars = newpars(bestpar[0+tcp*7],bestpar[2+tcp*7],bestpar[3+tcp*7],bestpar[1+tcp*7])
     bestTc0 = double(newpars[3])+2440000.D
     bestTe0 = separs(bestpar[0+tcp*7],bestpar[2+tcp*7],bestpar[3+tcp*7],bestpar[1+tcp*7])+2440000.D; secondary eclipse
     ; (2) Tc from best fitted parameters, for each transit
     period = bestpar[0+tcp*7]
     diffs = ceil((jdtoday-bestTc0)/period)+indgen(ntran) ; n, n+1, n+2 ... n+ntran-1
     bestTcs = bestTc0 + double(double(diffs)*period) ; transit time after jdtoday, jdtoday+period, jdtoday+2*period...
     bestTes = bestTe0 + double(double(diffs)*period); secondary eclipse
     ; (3) Tc0s sampling distribution
     nboot = n_elements(bootnewpar[*,0])
     tc0s = double(bootnewpar[*,3])+2440000.D
     te0s = double(bootse[*])+2440000.D
     for i=0,nboot-1 do begin
        ; wrap out-of-ordinary outliers to somewhere close to bestTc0
        ; unlikely to happen but...
        thisp = bootpar[i,0+tcp*7]
        tc0s[i] = restrict(tc0s[i],bestTc0-thisp/2.,bestTc0+thisp/2.) 
        te0s[i] = restrict(te0s[i],bestTe0-thisp/2.,bestTe0+thisp/2.) 
     endfor
     ; (4) For each transit, bootstrap Tc
     sigmaTcs = dblarr(ntran) ; WHAT WE WANT!
     sigmaTes = dblarr(ntran) ; WHAT WE WANT!
     for i=0,ntran-1 do begin
        ; For each tc0s[0:nboot-1], forward the system by n+i bootstrap periods
        tcs = tc0s + bootpar[*,0+tcp*7]*diffs[i]
        tes = te0s + bootpar[*,0+tcp*7]*diffs[i]
        if keyword_set(debug) then begin
           hist = histogram(tcs-2440000.D,binsize=0.05)
           plot,findgen(n_elements(hist))*0.05,hist,ps=10,/xs, $
                title='Sampling Distribution of Tc',xtit='Tc - '+strtrim(min(tcs),2)+' JD days',ytit='N'
           oplot,[bestTcs[i]-min(tcs)],[1.0],ps=4,symsize=3
           legend,['   Tc from best fit'],psym=[4],symsize=[3],charsize=1.2
           print,'max(Tc) - min(Tc): ',strtrim(max(tcs)-min(tcs),2)
           print,'Press any key to continue...'
           r=get_kbrd()
        endif 
        sigmaTcs[i] = robust_sigma(tcs)
        sigmaTes[i] = robust_sigma(tes)
     endfor 

     ; The old angle solution, unfinished and obsolete.
     ; Please ignore.
     ;; for i=0,ntran-1 do begin
     ;;    thistoday = jdtoday + i*bestpar[0] ; next transit after this date
     ;;    diffs = ceil((thistoday-tc0s)/bootpar[*,0]) 
     ;;    tcs = tc0s + double(diffs*bootpar[*,0]) ; between thistoday and thistoday+(each bootpar[*,0])
     ;;    ; begin angle analog analysis
     ;;    PI = 3.1415926536D
     ;;    ;maxp = max([bestpar[0],max(tcs-thistoday)]) ; proper period for wrapping
     ;;    tcangle = (tcs-thistoday)/bootpar[*,0]*2.0*PI
     ;;    meansin = mean(sin(tcangle))
     ;;    meancos = mean(cos(tcangle))
     ;;    angle = atan(meansin/meancos)
     ;;    case 1 of
     ;;       meansin lt 0. and meancos ge 0.: angle = angle+2.0*PI
     ;;       meancos le 0.: angle = angle+PI
     ;;       else: if angle ge 0.0 then angle = angle else print,'%% BOOTTRAN: Wrong Angle! Should never happen!'
     ;;    endcase
     ;;    ;meanTc = angle/2.0/PI*bootpar[*,0]+thistoday ; the real mean(Tc)
         
     ;;    ;normTcs = restrict(tcs-meanTc,-PI,PI)
     ;;    sigmaTcs[i] = robust_sigma(tcs)
     ;;    print,mean(tcs),format='(d)'
     ;; endfor

     if 1-keyword_set(noprint) then begin
        print,'Transit Mid-Point and Time Window for Planet # '+strtrim(Tc_planet,2)+':'
        print,'(Note: If your input JDs are not BJD, then Tc is reported in JD unless /regularjd and coords are given.)'
        print,'(Note: If your input JDs are BJD, then UT is calculated based on BJD unless you provide coords of target.)'
                                ;No, sigma(Tc) evolves with time!
        ;; print,'sigma(Tc): (+/-) '+strtrim(robust_sigma(bootnewpar[*,3]),2)+' days'
        caldat,jdtoday,m,d,y
        print,'Transit mid time after date (JD): '+strtrim(jdtoday,2) $
              +' (Y/M/D: '+strtrim(y,2)+'/'+strtrim(m,2)+'/'+strtrim(d,2)+')'
        for i=0,ntran-1 do begin
           if keyword_set(regularjd) and keyword_set(coords) then begin ; Needs to convert into BJD
              jdTcs = bestTcs[i] 
              bjdTcs = helio_jd(jdTes-2400000.d,coords[0],coords[1])+2400000.d
              jdTes = bestTes[i] 
              bjdTes = helio_jd(jdTes-2400000.d,coords[0],coords[1])+2400000.d
           endif else begin ; Needs to convert BJD into JD if given coords
              if keyword_set(coords) then begin
                 bjdTcs = bestTcs[i]
                 jdTcs = bjdTcs - helio_jd(bjdTcs-2400000.d,coords[0],coords[1],/time_diff)/86400.0d
                 bjdTes = bestTes[i]
                 jdTes = bjdTes - helio_jd(bjdTes-2400000.d,coords[0],coords[1],/time_diff)/86400.0d
              endif else begin ; nothing happens, Tc is not truly in BJD (if input is not BJD) or reported UT will have up to few minutes error (if input is BJD)
                 jdTcs = bestTcs[i]
                 bjdTcs = bestTcs[i]
                 jdTes = bestTes[i]
                 bjdTes = bestTes[i]
                 if keyword_set(regularjd) then print,'!!! WARNING: Tc is actually in regular JD since coords are not provided.'
              endelse
           endelse 
           caldat,jdTcs,mon,day,year,hour,min,sec
           print,format='("Tc # ", I1 ,": sigma(Tc) (day) ", d0, ", Tc (BJD)", d14.5,", (UT in Y/M/D H:M:S) ",'+ $
                 ' I4, "/", I02, "/", I02, " ", I02, ":", I02, ":", d04.1)', $
                 i+1, sigmaTcs[i], bjdTcs, year,mon,day,hour,min,sec
        endfor
        print,''
     endif 
  endif

  if keyword_set(bootoffset) then begin
     if datatype(bootoffset) ne 'STR' then begin
        if 1-keyword_set(noprint) then print,'Offsets to Telescope # 1:'
        if 1-keyword_set(noprint) then print,'!!! WARNING: Offsets listed here are not best-fit but the mean. '
        if 1-keyword_set(noprint) then print,'             Perform a rv_fit_mp to get the best-fit offsets.'
        noffset = n_elements(bootoffset[0,*])
        if 1-keyword_set(noprint) then begin
           for i=0,noffset-1 do begin
              print,'Telescope # '+strtrim(i+2,2)+': ' $
                    +strtrim(mean(bootoffset[*,i]),2) $
                    +'  +/-  '+strtrim(robust_sigma(bootoffset[*,i]),2)+' m/s'
           endfor 
           print,''
        endif 
     endif
  endif 

;--------------------------------------------------------------------------------------------  
; Save the work

                                ; DATA structure
  if n_elements(tve1) ge 1 then begin
     case 1 of                  ; Determine number of input parameters
        1-keyword_set(tve2): ntele=1
        1-keyword_set(tve3): ntele=2
        1-keyword_set(tve4): ntele=3
        1-keyword_set(tve5): ntele=4
        1-keyword_set(tve6): ntele=5
        else: ntele=6
     endcase  
     if 1-keyword_set(tele) then tele='NA'
     case 1 of
        ntele eq 1 : data = {tve1:tve1,tele:tele}
        ntele eq 2 : data = {tve1:tve1,tve2:tve2,tele:tele}
        ntele eq 3 : data = {tve1:tve1,tve2:tve2,tve3:tve3,tele:tele}
        ntele eq 4 : data = {tve1:tve1,tve2:tve2,tve3:tve3,tve4:tve4,tele:tele}
        ntele eq 5 : data = {tve1:tve1,tve2:tve2,tve3:tve3,tve4:tve4,tve5:tve5,tele:tele}
        else : data = {tve1:tve1,tve2:tve2,tve3:tve3,tve4:tve4,tve5:tve5,tve6:tve6,tele:tele}
     endcase
  endif 
                               
                                ; BOOT structure
  nplanet = n_elements(bestpar)/7
  if n_elements(bootnewpar) lt 1 then begin
                                ; Transit-related Parameters
     bootnewpar = 'Was Not Provided'
     bootse = 'Was Not Provided'
     Tc_planet = 'Need bootnewpar'
     bestTc = 'Need bootnewpar'
     sigma_Tc = 'Need bootnewpar'
     bestTe = 'Need bootnewpar'
     sigma_Te = 'Need bootnewpar'

  endif else begin
     bestTc = bestTcs
     sigma_Tc = sigmaTcs
     bestTe = bestTes
     sigma_Te = sigmaTes
  endelse 
  if n_elements(bootoffset) lt 1 then bootoffset = 'Not Multi-telescope Situation or NA'
  if n_elements(sigma_m) lt 1 then begin
     thissigma_m = 'NA'
     sigma_massf = 'Need sigma_m'
     sigma_msini = 'Need sigma_m'
     sigma_a = 'Need sigma_m'
  endif else begin
     thissigma_m = double(sigma_m)      ; protect sigma_m
  endelse 
  if n_elements(mstar) lt 1 then begin
     thismstar = 'NA'
     massfunc = 'Need mstar'
     msini = 'Need mstar'
     a = 'Need mstar'
     sigma_massf = 'Need mstar'
     sigma_msini = 'Need mstar'
     sigma_a = 'Need mstar'
  endif else begin
     thismstar = double(mstar)          ; protect mstar
  endelse 

  boot = ''
  boot = {nplanet:nplanet, $    ; Number of planets
          bestpar:bestpar, $    ; Best Fit Parameters, output in format of rvlin package:
          ;par_def:['Period [day]','Tp [JD-2440000]','e','omega [deg]','K [m/s]','gamma [m/s]','dv/dt [m/s/day]']
          sigma_par:sigma_par, $        ; Uncertainties on bestpar
          bootpar:bootpar, $            ; bootstrap replicates of bestpar
          bootnewpar:bootnewpar, $      ; bootstrap replicates of newpars (period,ecosom,esinom,Tc)
          bootse:bootse, $              ; bootstrap replicates of secondary eclipse times
          bootoffset:bootoffset, $      ; bootstrap replicates of telescope offsets
          Tc_planet:Tc_planet, $        ; Tc is calculated for which planet?
          bestTc:bestTc, $              ; Best fit Tc
          sigma_Tc:sigma_Tc, $          ; uncertainties on Tc
          bestTe:bestTe, $              ; Best fit Te
          sigma_Te:sigma_Te, $          ; uncertainties on Te
          mstar:thismstar, $            ; stellar mass
          sigma_m:thissigma_m, $        ; sigma of stellar mass
          massfunc:massfunc, $          ; mass function, in grams
          msini:msini, $                ; planet masses lower limit m*sin(i), in Jupiter Mass
          a:a, $                        ; semi-major axis, in AU
          sigma_massf:sigma_massf, $    ; uncertainties on mass function
          sigma_msini:sigma_msini, $    ; uncertainties on m*sin(i)
          sigma_a:sigma_a $             ; uncertainties on semi-major axes
         }
  
  if 1-keyword_set(nosave) then begin
     if 1-keyword_set(bootfile) then bootfile='temp-bootpars.sav'
     date=systime(0,/utc)
     save,date,data,boot,file=bootfile
     if 1-keyword_set(noprint) then print,'%% BOOTTRAN: Bootstrap results saved in: '+bootfile
  endif 
  
END 
