function newpars, inper, inecc, inom, intp
  
  ;converts P, e, om, and T_p to a more orthogonal set
  ;       P, e*cos(om), e*sin(om), and T_t, the time of "transit"
  ;   searches in these parameters may converge faster and be more stable.

  ;See Murray and Dermott (Cambridge University Press) for equations

  ;Omega is expected in degrees.

  if n_elements(inper) eq 0 then begin
    print, "Usage: "
    print, "n=newpars(per,ecc,om,tp)"
    print, "n contains p, ecos(om), esin(om), T_c"
    return, 0
  endif
  if n_elements(inecc) eq 0 then begin
    per = double(inper.per)
    ecc = double(inper.ecc)
    om = double(inper.om*!dtor)
    tp = double(inper.t0)
  endif else begin
    ecc = double(inecc)
    per = double(inper)
    om = double(inom*!dtor)
    tp = double(intp)
  endelse

  ecosom = ecc*cos(om)
  esinom = ecc*sin(om)

  f = !pi/2.-om
  E = 2*atan(sqrt((1-ecc)/(1+ecc))*sin(f/2.), cos(f/2.))
  n = 2.*!dpi/per
  tt = (E-ecc*sin(E))/n + tp

  return, [[reform(per)], [reform(ecosom)], [reform(esinom)], [reform(tt)]]

end
