function separs, inper, inecc, inom, intp
  
  ;converts P, e, om, and T_p to a more orthogonal set
  ;        the time of second eclipse
  ;   searches in these parameters may converge faster and be more stable.

  ;See Murray and Dermott (Cambridge University Press) for equations

  ;Omega is expected in degrees.

  if n_elements(inper) eq 0 then begin
    print, "Usage: "
    print, "n=separs(per,ecc,om,tp)"
    print, "n contains second eclipsing time"
    return, 0
  endif
  if n_elements(inecc) eq 0 then begin
    per = double(inper.per)
    ecc = double(inper.ecc)
    om = double(inper.om*!dtor)
    tp = double(inper.t0)
  endif else begin
    ecc = double(inecc)
    per = double(inper)
    om = double(inom*!dtor)
    tp = double(intp)
  endelse

  ecosom = ecc*cos(om)
  esinom = ecc*sin(om)

  f = 3*!pi/2.-om
  E = 2*atan(sqrt((1-ecc)/(1+ecc))*sin(f/2.), cos(f/2.))
  n = 2.*!dpi/per
  se = (E-ecc*sin(E))/n + tp

  return, se

end
